## curveToGeom.py
## written by Andrew Tubelli

import maya.cmds as cmds
import maya.mel as mel
import math

# COMMON BUTTON GUI
class AT_OptionsWindow(object):
    """A base class for an options window"""
    @classmethod
    def showUI(cls):
        """A function to instantiate the options window"""
        win = cls()
        win.create()
        return win
    def __init__(self):
        """Initialize common data attributes"""
        ## unique window handle
        self.window = 'at_optionsWindow'
        ## window title
        self.title = 'Options Window'
        ## window size
        self.size = (400, 200)
        ## specify whether the tool/action toggle is supported
        self.supportsToolAction = False
        ## name to display on the action button
        self.actionName = 'Apply and Close'
    def create(self):
        """Draw the window"""
        # delete the window if its handle exists
        if cmds.window(self.window, exists=True):
            cmds.deleteUI(self.window, window=True)
        # initialize the window
        self.window = cmds.window(
            self.window,
            title=self.title,
            widthHeight=self.size,
            menuBar=True
        )
        # main form for the window
        self.mainForm = cmds.formLayout(nd=100)
        # create common buttons
        self.commonMenu()
        self.commonButtons()
        # see getOptionBox.mel for why we implement this layout pattern to emulate Maya's option boxes
        self.optionsBorder = cmds.tabLayout(
            scrollable=True,
            tabsVisible=False,
            height=1,
            childResizable=True
        )
        cmds.formLayout(
            self.mainForm, e=True,
            attachForm=(
                [self.optionsBorder,'top',0],
                [self.optionsBorder,'left',2],
                [self.optionsBorder,'right',2]
            ),
            attachControl=(
                [self.optionsBorder,'bottom',5,self.closeBtn]
            )
        )
        # form to attach controls in displayOptions()
        self.optionsForm = cmds.formLayout(nd=100)
        self.displayOptions()
        # show the window
        cmds.showWindow()    
    def commonMenu(self):
        self.editMenu = cmds.menu(l='Edit')
        self.editMenuReset = cmds.menuItem(l='Delete Geometry and Reset Settings', command=self.editMenuResetCmd)
        self.helpMenu = cmds.menu(l='Help')
        self.helpMenuItem = cmds.menuItem(l='Instructions',command=self.helpMenuCmd)   
    def helpMenuCmd(self, *args):
        cmds.confirmDialog(
            t='Instructions',
            m='You must first select all curves that will be turned into geometry. If you wish to specify a name for '
              +'the surface, type that name in the \'Replace geometry:\' prompt. Then click the \'Create geometry\' button.\n\n'
              +'Next, refine the mesh by settings the controls listed under Step 2. The mesh is a polygon object that has been '
              +'converted from an nParticle object. Mesh Spacing controls how fine the mesh is. A smaller value results in a '
              +'finer mesh with more faces. Blobby Radius controls how large the output blobby particles are, and as a result, '
              +'the diameter of the mesh along its entire length. Smoothing controls how smooth the mesh is. Radius controls the '
              +'initial radius of the nParticles. This is different from the blobby radius, which additionally controls the radius '
              +'after the particles have been converted to polygons.\n\n'
              +'To delete all geometry and start over again, go to Edit > Delete Geometry and Reset Settings.\n\n'
              +'Choose \'Finalize and Close\' if you are happy with the geometry and want to remove history on the object. '
              +'If you still require control over the mesh, choose \'Close\' instead. For further access to the controls, go into '
              +'the attribute editor for the nParticleShape object, under the Output Mesh tab.',
            b='Close',
            p='at_optionsWindow',
            icn='information'
        )
    def editMenuResetCmd(self, *args):
        # check if exists...
        polys=cmds.filterExpand(sm=12)
        try:
            inputText in polys
        except:
            pass
        else:
            cmds.delete(inputText,'nucleus1',inputText+'_nParticle')
        AT_CurvesToGeomWindow.showUI()
    def finBtnCmd(self, *args):
        #Delete history and remove particles (not nucleus)
        cmds.delete(inputText, ch=True)
        self.closeBtnCmd()
    def closeBtnCmd(self, *args):
        """Close window"""
        cmds.deleteUI(self.window, window=True)
    def commonButtons(self):
        """Create common buttons for all option boxes"""
        self.commonBtnSize = (self.size[0], 26)
        self.finBtnCreate()
        self.closeBtn = cmds.button(
            label='Close',
            height=self.commonBtnSize[1],
            command=self.closeBtnCmd
        )
        cmds.formLayout(
            self.mainForm, e=True,
            attachForm=(
                [self.finBtn,'left',5],
                [self.finBtn,'bottom',5],
                [self.closeBtn,'bottom',5],
                [self.closeBtn,'right',5]
            ),
            attachControl=(
                [self.closeBtn,'left',5,self.finBtn]
            ),
            attachPosition=(
                [self.finBtn,'right',0,49],
                [self.closeBtn,'left',0,50]
            ),
            attachNone=(
                [self.closeBtn,'top']
            )
        )
    def finBtnCreate(self):
        self.finBtn = cmds.button(
            'finBut',
            label='Finalize and Close',
            height=self.commonBtnSize[1],
            command=self.finBtnCmd,
            en=False
        )
    def finBtnEnable(self):
        cmds.button('finBut', e=True, en=True)
    def displayOptions(self):
        """Override this method to display options controls"""
        pass

# ---------------------------------------------------------------------------        

# CONTENT GUI
class AT_CurvesToGeomWindow(AT_OptionsWindow):
    """A class for a simple window to create polygon primitives"""
    def __init__(self):
        """Initialize base class and override data attributes"""
        AT_OptionsWindow.__init__(self)
        # overridden from base class
        self.title = 'Turn Curves Into Geometry'
        # overridden from base class
        self.actionName = 'Create'
    
    def displayOptions(self):
        # Create a column layout
        cmds.columnLayout()
        # Create a button that calls the curveToGeom() function
        cmds.text(wordWrap=True, w=400, align='left',
          label='STEP 1: CREATE GEOMETRY')
        cmds.separator(height=10, style='none')
        cmds.text(wordWrap=True, w=self.size[0]-50, align='left',
          label='Please select all curves to be turned into geometry and then select'
          +' the \'Create geometry\' button below.')
        cmds.separator(height=10, style='none')
        cmds.textFieldGrp('renameTxtFld', label='Rename geometry: ')
     
        cmds.separator(height=10, style='none')        
        self.geomBtn = cmds.button(
            'createGeomBtn',
            label='Create geometry',
            width=400,
            height=self.commonBtnSize[1],
            command=self.createGeom,
            bgc=[0.5,0.5,1]
        )
        
        cmds.separator(height=25, width=400, style='in')
        
        cmds.columnLayout()
        cmds.text(wordWrap=True, w=400, align='left',label='STEP 2: REFINE GEOMETRY')
        cmds.separator(height=10, style='none')
        cmds.attrFieldSliderGrp('grp1', l='Mesh Spacing', min=0.10, max=0.5, pre=2, en=False)
        cmds.attrFieldSliderGrp('grp2', l='Blobby Radius', min=0.1, max=2.0, pre=2, en=False)
        cmds.attrFieldSliderGrp('grp3', l='Smoothing', min=0, max=40, pre=2, en=False)
        cmds.attrFieldSliderGrp('grp4', l='Radius', min=0, max=1, pre=2, en=False)
    
    def createGeom(self, *args):
        global inputText
        inputText = cmds.textFieldGrp('renameTxtFld', q=True, text=True)
        self.curveToGeom(inputText)
        self.activate_ctrls(inputText)
    
    def activate_ctrls(self, inputText):
        cmds.columnLayout()
        cmds.textFieldGrp('renameTxtFld', e=True, en=False)
        cmds.button('createGeomBtn', e=True, en=False, bgc=[0.3,0.3,0.3])
        cmds.attrFieldSliderGrp('grp1', e=True, at=inputText+'_nParticleShape'+'.meshTriangleSize', en=True)
        cmds.attrFieldSliderGrp('grp2', e=True, at=inputText+'_nParticleShape'+'.blobbyRadiusScale', en=True)
        cmds.attrFieldSliderGrp('grp3', e=True, at=inputText+'_nParticleShape'+'.meshSmoothingIterations', en=True)
        cmds.attrFieldSliderGrp('grp4', e=True, at=inputText+'_nParticleShape'+'.radius', en=True, cc='cmds.currentTime(0)')
        cmds.setAttr(inputText+'_nParticleShape'+'.meshTriangleSize', 0.23)
        self.finBtnEnable()
    
    def curveToGeom(self, inputText, *_):
        # Rebuild selected curves
        curve_list = cmds.ls(selection=True)
        
        if not curve_list:
            mel.eval('error "No curves have been selected. Please select curves before pushing this button."')
            return
        
        cmds.makeIdentity(apply=True, t=1, r=1, s=1, n=0)
        numCurves = len(curve_list)
        for x in curve_list:
            arcLen = cmds.arclen(x)
            numSpan = int(math.ceil(arcLen))*5
            cmds.rebuildCurve(x, rt=0, s=numSpan)
    
        # Obtain and store all the coordinates of the CVs for each curve in one variable
        CVcoords=[]
        for x in curve_list:
            numCV = cmds.getAttr(x+'.degree') + cmds.getAttr(x+'.spans')
            CVcoord = cmds.getAttr(x+'.cv[0:'+str(numCV)+']')
            for i in CVcoord:
                CVcoords.append(i)

        # Create nParticle at each coordinate
        #global nPartObj
        nPartObj = cmds.nParticle(p=CVcoords, n=inputText+'_nParticle')
        cmds.setAttr('nucleus1.gravity', 0)

        # Convert nParticle object to polygon
        cmds.select(nPartObj)
        nPartSurf = mel.eval('particleToPoly;')
        # print(cmds.ls(selection=True))
        cmds.rename(inputText)
        cmds.setAttr(nPartObj[1]+'.meshMethod', 3)

        # Set the width of the geometry (by changing the radius of the nParticles used to 
        # generate the geometry). This is hard-coded for now, but should be user-specified.
        cmds.setAttr(inputText+'_nParticleShape'+'.radius', 0.3)
        cmds.currentTime(0)

AT_CurvesToGeomWindow.showUI()